<?php
/**
 * Register custom dynamic blocks for Filr.
 *
 * @package Filr
 */

namespace filr;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class FILR_Blocks {
	private static $instance = null;

	public static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Constructor
	 */
	private function __construct() {
		add_action( 'init', array( $this, 'register_blocks' ) );
		add_action( 'enqueue_block_editor_assets', array( $this, 'enqueue_block_editor_assets' ) );
		add_action( 'init', array( $this, 'register_meta' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'add_frontend_css' ) );
	}

	/**
	 * Enqueue Filr Table View CSS for frontend
	 */
	public function add_frontend_css() {
		if ( is_admin() ) {
			return;
		}

		wp_enqueue_style(
			'filr-table-view',
			FILR_URL . '/assets/css/filr-table-view.css',
			array(),
			FILR_VERSION
		);
	}

	public function register_blocks() {
		// Download Button
		register_block_type(
			FILR_PATH . '/src/blocks/filr-download-button',
			array(
				'render_callback' => function ( $attributes, $content, $block ) {
					global $post;
					$download_url = get_post_meta( $post->ID, 'file-download', true );
					if ( ! $download_url ) {
						$download_url = '#';
					}
					return '<div class="wp-block-button"><a class="wp-block-button__link wp-element-button" href="' . esc_url( $download_url ) . '">' . esc_html__( 'Download', 'filr' ) . '</a></div>';
				},
			)
		);
		// File Extension
		register_block_type(
			FILR_PATH . '/src/blocks/filr-file-extension',
			array(
				'render_callback' => function ( $attributes, $content, $block ) {
					global $post;
					$meta = get_post_meta( $post->ID, 'file-upload', true );
					$ext  = '';
					if ( $meta ) {
						$arr = maybe_unserialize( $meta );
						if ( count( $arr['files'] ) > 1 ) {
							return 'zip';
						}
						if ( isset( $arr['files'][0]['extension'] ) ) {
							$ext = esc_html( $arr['files'][0]['extension'] );
						}
					}
					return $ext;
				},
			)
		);
		// File Size
		register_block_type(
			FILR_PATH . '/src/blocks/filr-file-size',
			array(
				'render_callback' => function ( $attributes, $content, $block ) {
					global $post;
					$meta = get_post_meta( $post->ID, 'file-upload', true );
					$size = '';
					if ( $meta ) {
						$arr = maybe_unserialize( $meta );
						if ( isset( $arr['files'][0]['size2'] ) ) {
							$size = esc_html( $arr['files'][0]['size2'] );
						}
					}
					return $size;
				},
			)
		);
		// Version
		register_block_type(
			FILR_PATH . '/src/blocks/filr-version',
			array(
				'render_callback' => function ( $attributes, $content, $block ) {
					global $post;
					$version = get_post_meta( $post->ID, 'version', true );
					return $version ? esc_html( $version ) : '';
				},
			)
		);
		// File Date
		register_block_type(
			FILR_PATH . '/src/blocks/filr-file-date',
			array(
				'render_callback' => function ( $attributes, $content, $block ) {
					global $post;
					$meta = get_post_meta( $post->ID, 'file-upload', true );
					$date = '';
					if ( $meta ) {
						$arr = maybe_unserialize( $meta );
						if ( isset( $arr['files'][0]['date'] ) ) {
							$timestamp = strtotime( $arr['files'][0]['date'] );
							$date = date_i18n( get_option( 'date_format' ), $timestamp );
						}
					}
					return $date;
				},
			)
		);
		// Expire After Downloads
		register_block_type(
			FILR_PATH . '/src/blocks/filr-expire-download',
			array(
				'render_callback' => function ( $attributes, $content, $block ) {
					global $post;
					$expire = get_post_meta( $post->ID, 'expire-download', true );
					return $expire ? esc_html( $expire ) : '';
				},
			)
		);
		// Expire Date
		register_block_type(
			FILR_PATH . '/src/blocks/filr-expire-date',
			array(
				'render_callback' => function ( $attributes, $content, $block ) {
					global $post;
					$expire = get_post_meta( $post->ID, 'expire-date', true );
					return $expire ? esc_html( $expire ) : '';
				},
			)
		);
	}

	public function enqueue_block_editor_assets() {
		$asset_file = FILR_PATH . '/assets/gutenberg/index.asset.php';
		$script_url = FILR_URL . '/assets/gutenberg/index.js';

		if ( file_exists( $asset_file ) ) {
			$asset = include $asset_file;
			wp_enqueue_script(
				'filr-blocks',
				$script_url,
				$asset['dependencies'],
				$asset['version'],
				true
			);
		}
	}

	public function register_meta() {
		register_post_meta(
			'filr',
			'file-upload',
			array(
				'show_in_rest'  => true,
				'single'        => true,
				'type'          => 'string',
				'auth_callback' => function () {
					return current_user_can( 'edit_posts' );
				},
			)
		);

		register_post_meta(
			'filr',
			'file-download',
			array(
				'show_in_rest'  => true,
				'single'        => true,
				'type'          => 'string',
				'auth_callback' => function () {
					return current_user_can( 'edit_posts' );
				},
			)
		);

		register_post_meta(
			'filr',
			'version',
			array(
				'show_in_rest'  => true,
				'single'        => true,
				'type'          => 'string',
				'auth_callback' => function () {
					return current_user_can( 'edit_posts' );
				},
			)
		);

		register_post_meta(
			'filr',
			'expire-download',
			array(
				'show_in_rest'  => true,
				'single'        => true,
				'type'          => 'string',
				'auth_callback' => function () {
					return current_user_can( 'edit_posts' );
				},
			)
		);

		register_post_meta(
			'filr',
			'expire-date',
			array(
				'show_in_rest'  => true,
				'single'        => true,
				'type'          => 'string',
				'auth_callback' => function () {
					return current_user_can( 'edit_posts' );
				},
			)
		);
	}
}
